type ConcatPath<T extends string, P extends string> = T extends ''
  ? P
  : `${T}.${P}`

type Length<T extends any[]> = T extends { length: infer L } ? L : never

type BuildTuple<L extends number, T extends any[] = []> = T extends {
  length: L
}
  ? T
  : BuildTuple<L, [...T, any]>

type Subtract<A extends number, B extends number> = BuildTuple<A> extends [
  ...infer U,
  ...BuildTuple<B>
]
  ? Length<U>
  : never

export type AllPaths<
  T,
  P extends string = '',
  MaxDepth extends number = 10
> = T extends (infer U)[]
  ?
      | P
      | `${P}[${number}]`
      | AllPaths<U, `${P}[${number}]`, Subtract<MaxDepth, 1>>
  : T extends BaseDeepMap
  ? MaxDepth extends 0
    ? never
    : {
        [K in keyof T]-?: K extends string | number
          ?
              | AllPaths<T[K], ConcatPath<P, `${K}`>, Subtract<MaxDepth, 1>>
              | (P extends '' ? never : P)
          : never
      }[keyof T]
  : P

type IsNumber<T extends string> = T extends `${number}` ? true : false

type NestedObjKey<T, P> = P extends `${infer A}.${infer B}`
  ? A extends keyof T
    ? FromPath<NonNullable<T[A]>, B>
    : never
  : never

type NestedArrKey<T, P> = P extends `${infer A}[${infer I}]${infer R}`
  ? [A, NonNullable<T[Extract<A, keyof T>]>, IsNumber<I>] extends [
      keyof T,
      (infer Item)[],
      true
    ]
    ? R extends `.${infer NewR}`
      ? FromPath<Item, NewR>
      : Item
    : never
  : never

export type FromPath<T, P> = T extends unknown
  ? NestedArrKey<T, P> extends never
    ? NestedObjKey<T, P> extends never
      ? P extends keyof T
        ? T[P]
        : never
      : NestedObjKey<T, P>
    : NestedArrKey<T, P>
  : never

export type BaseDeepMap = Record<string, unknown>

/**
 * Get a value by object path. `undefined` if key is missing.
 *
 * ```
 * import { getPath } from 'nanostores'
 *
 * getPath({ a: { b: { c: ['hey', 'Hi!'] } } }, 'a.b.c[1]') // Returns 'Hi!'
 * ```
 *
 * @param obj Any object.
 * @param path Path splitted by dots and `[]`. Like: `props.arr[1].nested`.
 * @returns The value for this path. Undefined if key is missing.
 */
export function getPath<T extends BaseDeepMap, K extends AllPaths<T>>(
  obj: T,
  path: K
): FromPath<T, K>

/**
 * Set a deep value by key. Initialized arrays with `undefined`
 * if you set arbitrary length.
 *
 * ```
 * import { setPath } from 'nanostores'
 *
 * setPath({ a: { b: { c: [] } } }, 'a.b.c[1]', 'hey')
 * // Returns `{ a: { b: { c: [undefined, 'hey'] } } }`
 * ```
 *
 * @param obj Any object.
 * @param path Path splitted by dots and `[]`. Like: `props.arr[1].nested`.
 * @returns The new object.
 */
export function setPath<T extends BaseDeepMap, K extends AllPaths<T>>(
  obj: T,
  path: K,
  value: FromPath<T, K>
): T
